@@ -1,5 +1,5 @@
 # MATT - A Framework For Modifying And Testing Topologies
-# Copyright (C) 2020 Jeff Raffael Gower
+# Copyright (C) 2021 Jeff Raffael Gower
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -43,22 +43,32 @@ config = configparser.ConfigParser()
 config_path = os.path.join(root_folder, "config.ini")
 
 
-@app.route("/")
+@app.route("/", methods=["GET"])
 def home():
+    """
+    Returns the index.html which acts as the frontend
+    :return: response
+    """
     session["trees"] = []
-    # TODO set options from file
-    response = make_response(render_template("index.html"))
+    version = ""
+    with open("../version.txt", "r") as vf:
+        version = vf.read()
+    response = make_response(render_template("index.html", data=version))
     response.headers["Cache-Control"] = "no-store"
     return response
 
 
 @app.route("/get-options", methods=["GET"])
 def get_options():
+    """
+    Returns the options from the config.ini
+    :return: response
+    """
     config.read(config_path)
     while not config.has_section("Options"):
         set_default_config()
     else:
-        enable_lengths = config.getboolean("Options", "enable-lengths")
+        align_labels = config.getboolean("Options", "align-labels")
         working_directory = config.get("Options", "working-directory")
         dna_protein = config.get("Options", "dna-protein")
         dna_bsr = config.get("Options", "dna-bsr")
@@ -68,7 +78,7 @@ def get_options():
         protein_pmm = config.get("Options", "protein-pmm")
         protein_aaf = config.get("Options", "protein-aaf")
         protein_rhas = config.get("Options", "protein-rhas")
-    options = dumps({"enable_lengths": enable_lengths,
+    options = dumps({"align_labels": align_labels,
                      "working_directory": working_directory,
                      "dna_protein": dna_protein,
                      "dna_bsr": dna_bsr,
@@ -78,6 +88,10 @@ def get_options():
                      "protein_pmm": protein_pmm,
                      "protein_aaf": protein_aaf,
                      "protein_rhas": protein_rhas})
+    session["working-directory"] = config.get("Options", "working-directory")
+    if not os.path.isdir(session["working-directory"]):
+        session["working-directory"] = root_folder
+        config.set("Options", "working-directory", root_folder)
     response = make_response(options)
     response.headers["Cache-Control"] = "no-store"
     return response
@@ -85,7 +99,12 @@ def get_options():
 
 @app.route("/download/<tree_id>", methods=["GET"])
 def download(tree_id):
-    path = os.path.join(root_folder, "tmp", "download.nck")
+    """
+    Downloads the given tree
+    :param tree_id: the tree id in the database
+    :return: tree in newick format
+    """
+    path = os.path.join(session["working-directory"], "download.nck")
     file = open(path, "w")
     conn = sqlite3.connect(root_folder + 'trees.db')
     c = conn.cursor()
@@ -95,12 +114,16 @@ def download(tree_id):
     tree = Tree(tree_json).to_newick()
     file.write(tree + "\n")
     file.close()
-    return send_from_directory(os.path.join(root_folder, "tmp"), "download.nck", as_attachment=True,
+    return send_from_directory(os.path.join(session["working-directory"]), "download.nck", as_attachment=True,
                                attachment_filename=tree_description + ".nck")
 
 
 @app.route("/description", methods=["POST"])
 def description():
+    """
+    Sets the description of given tree
+    :return: response
+    """
     conn = sqlite3.connect(root_folder + 'trees.db')
     c = conn.cursor()
     form_id = request.form.get("id")
@@ -119,6 +142,10 @@ def description():
 
 @app.route("/load", methods=["POST", "GET"])
 def load():
+    """
+    Loads the tree from alignment and/or tree file or alters the tree by rehanging or rerooting using given parameters
+    :return: response
+    """
     model = None
     conn = sqlite3.connect(root_folder + 'trees.db')
     c = conn.cursor()
@@ -127,7 +154,7 @@ def load():
     while not config.has_section("Options"):
         set_default_config()
     else:
-        enable_lengths = config.getboolean("Options", "enable-lengths")
+        align_labels = config.getboolean("Options", "align-labels")
         working_directory = config.get("Options", "working-directory")
         dna_protein = config.get("Options", "dna-protein")
         if dna_protein == "dna":
@@ -152,12 +179,11 @@ def load():
             if protein_rhas != "-":
                 model += "+" + protein_rhas
     # TODO not POST/GET rather one or two form args or none if it is just a reload from saving options (TODO)
-    disable_testing = False
 
     if request.method == "POST":
         # TODO always delete tmp afterwards maybe??
-        # TODO here already without lengths too? Maybe, needs to be checked^^
-        alignment = tree = None
+        alignment = None
+        tree = None
 
         if request.form.get("example") is not None:
             example_alignment = open(os.path.join(root_folder, "static", "example.phy"), "r")
@@ -176,16 +202,17 @@ def load():
                 # tree_type = request.form.get("tree[name]").split(".")[-1]
 
         if alignment is not None and tree is not None:  # Case 1, alignment and tree given, default behaviour
-            with open(os.path.join(root_folder, "tmp", "alignment.phy"), "w") as alignment_file:
+            with open(os.path.join(session["working-directory"], "alignment.phy"), "w") as alignment_file:
                 alignment_file.write(alignment)
-            tree = Tree(tree, enable_lengths=enable_lengths).to_json()
+            tree = Tree(tree, align_labels=align_labels).to_json()
+            session["disable_testing"] = False
         elif alignment is not None:  # Case 2, only alignment given, construct ml-tree
-            with open(os.path.join(root_folder, "tmp", "alignment.phy"), "w") as alignment_file:
+            with open(os.path.join(session["working-directory"], "alignment.phy"), "w") as alignment_file:
                 alignment_file.write(alignment)
             if model is not None:
                 print("Starting IQTree. This could take some time!")
                 sp = subprocess.run([os.path.join(app_location, "bin", "iqtree"), "-s",
-                                     os.path.join(root_folder, "tmp", "alignment.phy"), "-m", model, "-redo"],
+                                     os.path.join(session["working-directory"], "alignment.phy"), "-m", model, "-redo"],
                                     capture_output=True)
                 print(sp)
                 if sp.returncode == 2:
@@ -193,52 +220,55 @@ def load():
             else:
                 print("Starting IQTree. This could take some time!")
                 sp = subprocess.run([os.path.join(app_location, "bin", "iqtree"), "-s",
-                                     os.path.join(root_folder, "tmp", "alignment.phy"), "-redo"], capture_output=True)
+                                     os.path.join(session["working-directory"], "alignment.phy"), "-redo"],
+                                    capture_output=True)
                 print(sp)
-            with open(os.path.join(root_folder, "tmp", "alignment.phy.treefile"), "r") as tree_file:
+            with open(os.path.join(session["working-directory"], "alignment.phy.treefile"), "r") as tree_file:
                 tree = tree_file.readline()
-            tree = Tree(tree[:-1], enable_lengths=enable_lengths).to_json()
-        elif tree is not None:  # Case 3, only tree given, disable testing and lengths
-            enable_lengths = False
-            tree = Tree(tree, enable_lengths=enable_lengths).to_json()
-            disable_testing = True
+            tree = Tree(tree[:-1], align_labels=align_labels).to_json()
+            session["disable_testing"] = False
+        elif tree is not None:  # Case 3, only tree given, disable testing
+            tree = Tree(tree, align_labels=align_labels).to_json()
+            session["disable_testing"] = True
     elif request.method == "GET":  # TODO post too?
         c.execute('SELECT json FROM trees WHERE id = ?', [session["tree"]])
         tree_json = c.fetchone()[0]
-        print(request.args)
         if len(request.args) == 0:
             json_args = None
-        if request.args.get("id") is not None:
-            json_args = [request.args.get("id")]
-        elif request.args.get("from") is not None:
-            json_args = [request.args.get("from"), request.args.get("to")]
-        else:
-            pass  # TODO
-        if request.args.get("current") is not None:
-            current = request.args.get("current")
-        else:
-            current = None
-        if enable_lengths:
-            # TODO same for case 2, provide options there too
-            tree = Tree(tree_json, json_args, enable_lengths).to_newick(True)
-            path = os.path.join(root_folder, "tmp", "tree.nck")
+        if request.args.get("lengths") is not None:
+            tree = Tree(tree_json, align_labels=align_labels, enable_lengths=True).to_newick(True)
+            path = os.path.join(session["working-directory"], "tree.nck")
             file = open(path, "w")
             file.write(tree + "\n")
             file.close()
             print("Starting IQTree. This could take some time!")
-            sp = subprocess.run([os.path.join(app_location, "bin", "iqtree"), "-s", os.path.join(root_folder, "tmp",
-                                                                                                 "alignment.phy"),
-                                 "-te", path, "-nt", "4", "-m", model, "-redo"], capture_output=True)
+            sp = subprocess.run(
+                [os.path.join(app_location, "bin", "iqtree"), "-s", os.path.join(session["working-directory"],
+                                                                                 "alignment.phy"),
+                 "-te", path, "-nt", "4", "-m", model, "-redo"], capture_output=True)
             print(sp)
             if sp.returncode == 2:
                 print("WRONG DECISION DNA/PROTEIN")
-            with open(os.path.join(root_folder, "tmp", "alignment.phy.treefile"), "r") as tree_file:
+            with open(os.path.join(session["working-directory"], "alignment.phy.treefile"), "r") as tree_file:
                 tree = tree_file.readline()
-            tree = Tree(tree[:-1], enable_lengths=enable_lengths).to_json()
+            if request.args.get("lengths") == "enabled":
+                tree = Tree(tree[:-1], align_labels=align_labels, enable_lengths=True).to_json()
+            else:
+                tree = Tree(tree[:-1], align_labels=align_labels, enable_lengths=False).to_json()
         else:
-            tree = Tree(tree_json, json_args, enable_lengths).to_json()
-        if current is not None:
-            del session["trees"][int(current):]
+            if request.args.get("id") is not None:
+                json_args = [request.args.get("id")]
+            elif request.args.get("from") is not None:
+                json_args = [request.args.get("from"), request.args.get("to")]
+            else:
+                pass  # TODO
+            if request.args.get("current") is not None:
+                current = request.args.get("current")
+            else:
+                current = None
+            tree = Tree(tree_json, json_args, align_labels).to_json()
+            if current is not None:
+                del session["trees"][int(current):]
     else:
         pass  # TODO
 
@@ -252,7 +282,7 @@ def load():
     conn.close()
     response = make_response(dumps(trees))
     response.headers["Cache-Control"] = "no-store"
-    if disable_testing:
+    if session["disable_testing"]:
         response.headers["Testing"] = "disabled"
     else:
         response.headers["Testing"] = "enabled"
@@ -261,6 +291,11 @@ def load():
 
 @app.route("/reset/<tree_id>", methods=["GET"])
 def reset(tree_id):
+    """
+    Resets the session object
+    :param tree_id: id of the tree that should be saved in the session afterwards
+    :return: response
+    """
     session["trees"] = [tree_id]
     response = make_response("OK")
     response.headers["Cache-Control"] = "no-store"
@@ -269,13 +304,16 @@ def reset(tree_id):
 
 @app.route("/options", methods=["POST"])
 def options():
+    """
+    Sets the options and saves them to the config.ini
+    :return: response
+    """
     config.read(config_path)
     while not config.has_section("Options"):
         set_default_config()
     else:
-        config.set("Options", "enable-lengths", request.form.get("enable-lengths"))
-
-    print(request.form) # TODO
+        config.set("Options", "align-labels", request.form.get("align-labels"))
+        config.set("Options", "working-directory", request.form.get("working-directory"))
 
     if request.form.get("dna-protein") == "dna" or request.form.get("dna-protein") == "protein":
         config.set("Options", "dna-protein", request.form.get("dna-protein"))
@@ -291,7 +329,16 @@ def options():
         config.set("Options", "protein-rhas", request.form.get("protein-rhas"))
     with open(root_folder + "config.ini", "w") as config_file:
         config.write(config_file)
-    response = make_response("OK")  # TODO
+    session["working-directory"] = request.form.get("working-directory")
+    if not os.path.isdir(session["working-directory"]):
+        if session["working-directory"] == "":
+            response = make_response("OK")
+        else:
+            response = make_response("Invalid directory")
+        session["working-directory"] = root_folder
+        config.set("Options", "working-directory", root_folder)
+    else:
+        response = make_response("OK")
     # TODO PRINT NEW TREE (options to the whole data thingy too)
     response.headers["Cache-Control"] = "no-store"
     return response
@@ -299,15 +346,25 @@ def options():
 
 @app.route("/tests", methods=["POST"])
 def tests():
+    """
+    Tests the given trees
+    :return: response
+    """
     # TODO handle that testing is disabled but ppl still try to test
     snapshots = request.form.getlist("snapshots[]")
-    path = os.path.join(root_folder, "tmp", "tests.nck")
+    if len(snapshots) == 1:
+        if snapshots[0] == str(session["trees"][0]):
+            response = make_response("NO")
+            response.headers["Cache-Control"] = "no-store"
+            return response
+        else:
+            snapshots.append(str(session["trees"][0]))
+    path = os.path.join(session["working-directory"], "tests.nck")
     file = open(path, "w")
     conn = sqlite3.connect(root_folder + 'trees.db')
     c = conn.cursor()
     trees_json = c.execute("SELECT json FROM trees WHERE id IN ({})".format(','.join('?' for _ in snapshots)),
                            snapshots).fetchall()
-    # TODO USE REAL LENGTHS NO MATTER WHAT ENABLE_LENGTHS IS!
     for tree_json in trees_json[:-1]:
         tree = Tree(tree_json[0]).to_newick()
         file.write(tree)
@@ -318,7 +375,6 @@ def tests():
     while not config.has_section("Options"):
         set_default_config()
     else:
-        working_directory = config.get("Options", "working-directory")
         dna_protein = config.get("Options", "dna-protein")
         if dna_protein == "dna":
             dna_bsr = config.get("Options", "dna-bsr")
@@ -343,11 +399,12 @@ def tests():
                 model += "+" + protein_rhas
     print("Starting IQTree. This could take some time!")
     sp = subprocess.run(
-        [os.path.join(app_location, "bin", "iqtree"), "-s", os.path.join(root_folder, "tmp", "alignment.phy"), "-z",
-         path, "-n", "0", "-zb", "10000", "-zw", "-au", "-m", model, "-redo"]) # TODO capture_output=True
+        [os.path.join(app_location, "bin", "iqtree"), "-s", os.path.join(session["working-directory"], "alignment.phy"),
+         "-z",
+         path, "-n", "0", "-zb", "10000", "-zw", "-au", "-m", model, "-redo"])  # TODO capture_output=True
     print(sp)
     results = []
-    path = os.path.join(root_folder, "tmp", "alignment.phy.iqtree")
+    path = os.path.join(session["working-directory"], "alignment.phy.iqtree")
     file = open(path, "r")
     for line in file:
         if line.startswith(
@@ -359,11 +416,34 @@ def tests():
     response.headers["Cache-Control"] = "no-store"
     return response
 
+"""
+def compute_branch_lengths():
+    tree = Tree(tree_json, json_args, enable_lengths, align_labels).to_newick(True)
+    path = os.path.join(session["working-directory"], "tree.nck")
+    file = open(path, "w")
+    file.write(tree + "\n")
+    file.close()
+    print("Starting IQTree. This could take some time!")
+    sp = subprocess.run(
+        [os.path.join(app_location, "bin", "iqtree"), "-s", os.path.join(session["working-directory"],
+                                                                         "alignment.phy"),
+         "-te", path, "-nt", "4", "-m", model, "-redo"], capture_output=True)
+    print(sp)
+    if sp.returncode == 2:
+        print("WRONG DECISION DNA/PROTEIN")
+    with open(os.path.join(session["working-directory"], "alignment.phy.treefile"), "r") as tree_file:
+        tree = tree_file.readline()
+    tree = Tree(tree[:-1], enable_lengths=enable_lengths, align_labels=align_labels).to_json()
+"""
 
 def set_default_config():
+    """
+    Saves the default config to the config.ini
+    :return: None
+    """
     config["Options"] = {
-        'enable-lengths': 'false',
-        'working-directory': '-',
+        'align-labels': 'true',
+        'working-directory': '',
         'dna-protein': 'dna',
         'dna-bsr': 'GTR',
         'dna-bf': '-',
@@ -378,11 +458,19 @@ def set_default_config():
 
 
 def open_browser():
+    """
+    Opens the frontend after a two second delay
+    :return: None
+    """
     time.sleep(2)
     webbrowser.open("http://127.0.0.1:5000/")
 
 
 def main():
+    """
+    Main function that sets up the database and the config and opens the frontend
+    :return: None
+    """
     conn = sqlite3.connect(root_folder + 'trees.db')
     c = conn.cursor()
     c.execute('''DROP TABLE IF EXISTS trees''')
@@ -394,8 +482,16 @@ def main():
     if not os.path.exists(root_folder + "config.ini"):
         set_default_config()
 
-    if not os.path.exists(os.path.join(root_folder, "tmp")):
-        os.makedirs(os.path.join(root_folder, "tmp"))
+    config.read(config_path)
+    while not config.has_section("Options"):
+        set_default_config()
+
+    working_directory = config.get("Options", "working-directory")
+    if not os.path.isdir(working_directory):
+        working_directory = root_folder
+
+    if not os.path.exists(os.path.join(working_directory)):
+        os.makedirs(os.path.join(working_directory))
 
     thread = threading.Thread(target=open_browser)
     thread.start()
